import psycopg2
import datetime
from matplotlib import pyplot as plt
import matplotlib.dates as mdates
host = input("Insert host:")
dbname = input("Insert database name:")
user = input("Insert user:")
password = input("Insert password:")
remConn = psycopg2.connect("dbname=" + dbname + " user=" + user + " password=" + password + " host=" + host)
remCur = remConn.cursor()
locConn = psycopg2.connect("dbname=crelip user=crelip")
locCur = locConn.cursor()

#Vytvorit zoznam dobrych IP adries
#Aspon 1 dobry vysledok z ping
remCur.execute("CREATE TEMPORARY TABLE ping_ip AS(SELECT DISTINCT ip_addr from ping WHERE ping_ploss >= 0 AND ping_ploss < 100)")
#Aspon 1 dobry vysledok z topology
remCur.execute("CREATE TEMPORARY TABLE topology_ip AS(SELECT DISTINCT ip_addr FROM topology WHERE t_status = 'C')")
#Nerezervovane IP adresy
remCur.execute("CREATE TEMPORARY TABLE non_reserved_ip AS(SELECT h.ip_addr AS ip_addr FROM hosts h WHERE NOT (h.ip_addr <<= '0.0.0.0/8' OR h.ip_addr  <<= '0.0.0.0/32' OR h.ip_addr  <<= '10.0.0.0/8' OR h.ip_addr  <<= '100.64.0.0/10' OR h.ip_addr  <<= '127.0.0.0/8' OR h.ip_addr  <<= '169.254.0.0/16' OR h.ip_addr  <<= '172.16.0.0/12' OR h.ip_addr  <<= '192.0.2.0/24' OR h.ip_addr<<= '192.88.99.0/24' OR h.ip_addr  <<= '192.88.99.2/32' OR h.ip_addr <<= '192.168.0.0/16' OR h.ip_addr <<= '192.0.0.0/24' OR h.ip_addr  <<= '198.18.0.0/15' OR h.ip_addr  <<= '198.51.100.0/24' OR h.ip_addr <<= '203.0.113.0/24' OR h.ip_addr <<= '255.255.255.255/32'));")
#Prienik 3 predoslych tableov
remCur.execute("CREATE TEMPORARY TABLE good_ip AS(SELECT ip_addr FROM ping_ip INTERSECT SELECT ip_addr FROM topology_ip INTERSECT SELECT ip_addr FROM non_reserved_ip);")
print("Triedenie hotove")

#Vybrat vsetky udaje z ping a topology pre vsetky dobre IP adresy - berieme len z dni, za ktore existuju zaznamy v oboch tabulkach
remCur.execute("""SELECT AVG(p.ping_rttmin) AS ping_min, AVG(p.ping_rttavg) AS ping_avg, AVG(p.ping_rttmax) AS ping_max, AVG(t.t_roundtrip[array_upper(t.t_roundtrip, 1)]), DATE(p.ping_date), DATE(t.t_date)
	FROM ping p JOIN good_ip g ON p.ip_addr = g.ip_addr
	JOIN topology t ON p.ip_addr = t.ip_addr AND DATE(p.ping_date) = DATE(t.t_date)
	WHERE p.ping_ploss >= 0 AND p.ping_ploss < 100 AND t.t_status = 'C'
	GROUP BY DATE(p.ping_date), DATE(t.t_date)
	HAVING AVG(p.ping_rttmin) > 0 AND AVG(p.ping_rttavg) > 0 AND AVG(p.ping_rttmax) > 0
	ORDER BY DATE(p.ping_date) ASC""")
pingMin = []
pingAvg = []
pingMax = []
roundtripResult = []
date = []
minRatio = []
avgRatio = []
maxRatio = []
for record in remCur:
	pingMin.append(record[0])
	pingAvg.append(record[1])
	pingMax.append(record[2])
	roundtripResult.append(float(record[3]))
	date.append(record[4])
	minRatio.append(roundtripResult[-1] / pingMin[-1])
	avgRatio.append(roundtripResult[-1] / pingAvg[-1])
	maxRatio.append(roundtripResult[-1] / pingMax[-1])

plt.figure(figsize=(10, 6))
fig, ax1 = plt.subplots()
ax1.plot(date, minRatio, marker='o', linestyle='', color='r')

ax2 = ax1.twinx()
ax2.plot(date, avgRatio, marker='o', linestyle='', color='g')

ax3 = ax1.twinx()
ax3.plot(date, maxRatio, marker='o', linestyle='', color='b')

plt.xlabel('Date')
plt.ylabel('Values')

min_y = min(min(minRatio), min(avgRatio), min(maxRatio))
max_y = max(max(minRatio), max(avgRatio), max(maxRatio))
ax1.set_ylim(min_y, max_y)
ax2.set_ylim(min_y, max_y)
ax3.set_ylim(min_y, max_y)

# Formatting x-axis with date
plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
plt.gca().xaxis.set_major_locator(mdates.YearLocator())
plt.gcf().autofmt_xdate()
plt.show()
	
remCur.close()
remConn.close()
locCur.close()
locConn.close()
